@@ -1,20 +1,25 @@
 from traitlets import Unicode, Any, Dict, List, Bool, Int
 from pathlib import Path
+from natsort import natsorted
 
 import pandas as pd
 import geopandas as gpd
 import numpy as np
 import rasterio as rio
+from rasterio.windows import from_bounds
+from matplotlib.colors import to_rgba
 
 from .parameters import *
 from sepal_ui.model import Model
 from sepal_ui.scripts import gee
 from sepal_ui.scripts import utils as su
-from matplotlib.colors import to_rgba
-
+from sepal_ui.message import ms
+from sepal_ui import aoi
 
 import ee
 
+__all__ = ["ReclassifyModel"]
+
 
 class ReclassifyModel(Model):
     """
@@ -38,14 +43,16 @@ class ReclassifyModel(Model):
         dst_gee (str): the output assetId. default to
             :code:`dst_dir/f'{src_gee.stem}_reclass``
         remaped (int): state var updated each time an input is remapped
-        aoi_model (aoi.model): AOI model object to get an area of interest it
-             is selected
+        aoi_model (aoi.AoiModel): AOI model object to get an area of interest if one is selected
         folder(str, optional): the init GEE asset folder where the asset selector should start looking (debugging purpose)
+        enforce_aoi (bool, optional): either or not an aoi should be set to allow the reclassification
 
     Args:
         gee (bool): either or not to set :code:`gee` to True
         dst_dir (str): the destination forlder for outputs
         folder(str, optional): the init GEE asset folder where the asset selector should start looking (debugging purpose)
+        aoi_model (aoi.AoiModel, optional): the aoi model to link to the reclassify workflow
+        enforce_aoi (bool, optional): either or not an aoi should be set to allow the reclassification
     """
 
     # inputs
@@ -53,6 +60,7 @@ class ReclassifyModel(Model):
     band = Any(None).tag(sync=True)
     src_local = Any(None).tag(sync=True)
     src_gee = Any(None).tag(sync=True)
+    dst_class_file = Any(None).tag(sync=True)
     dst_dir = Any(None).tag(sync=True)
     gee = Bool(False).tag(sync=True)
 
@@ -68,51 +76,57 @@ class ReclassifyModel(Model):
 
     # Create a state var, to determine if an asset has been remaped
     remaped = Int(False).tag(sync=True)
+    table_created = Bool(False).tag(sync=True)
 
-    def __init__(self, gee=False, dst_dir=None, aoi_model=None, folder=None, **kwargs):
+    def __init__(
+        self,
+        gee=False,
+        dst_dir=Path.home(),
+        aoi_model=None,
+        folder=None,
+        save=True,
+        enforce_aoi=False,
+        **kwargs,
+    ):
 
         # init the model
         super().__init__(**kwargs)
 
         # save the folder where the results should be stored
         # only used for local export
-        self.dst_dir = Path(dst_dir) if dst_dir else Path.home()
+        self.dst_dir = Path(dst_dir)
 
         # save relation with gee
         self.gee = gee
-        self.aoi_model = aoi_model
 
         if self.gee:
             su.init_ee()
-            self.folder = folder if folder else ee.data.getAssetRoots()[0]["id"]
+
+        if self.gee:
+            self.folder = folder or ee.data.getAssetRoots()[0]["id"]
+        else:
+            self.folder = None
+
+        self.aoi_model = aoi_model
+
+        # aoi_model and reclassify model must be aligned when it comes to gee
+        if self.aoi_model:
+            if self.aoi_model.ee != self.gee:
+                raise Exception(
+                    "Both aoi_model.gee and self.gee parameters has to be equals."
+                    + f"Received {self.aoi_model.ee} for aoi_model and {self.gee} for reclassify_model."
+                )
+
+        self.enforce_aoi = enforce_aoi
+
+        # set if the model need to save by default
+        self.save = save
 
         # memory outputs
         self.dst_local_memory = None
         self.dst_gee_memory = None
 
-    def save_matrix(self, filename):
-        """
-        Save the matrix i a csv file
-
-        Return:
-            self
-        """
-
-        if not len(self.matrix):
-            return self
-
-        df = pd.Dataframe(
-            {
-                "src": [c for c in self.matrix.keys()],
-                "dst": [c for c in self.matrix.values()],
-            }
-        )
-        df.to_csv(filename)
-
-        return self
-
-    @staticmethod
-    def get_classes(file):
+    def get_classes(self):
         """
         Extract the classes from the class file. The class file need to be compatible with the reclassify tool i.e. a table file with 3 headerless columns using the following format: 'code', 'desc', 'color'. Color need to be set in hexadecimal to be read else black will be used.
 
@@ -124,6 +138,11 @@ class ReclassifyModel(Model):
                 {code: (name, color)}
         """
 
+        file = self.dst_class_file
+
+        if not file:
+            raise AttributeError("missing file")
+
         path = Path(file)
         if not path.is_file():
             raise Exception(f"{file} is not existing")
@@ -186,14 +205,14 @@ class ReclassifyModel(Model):
         @su.need_ee
         def _ee_image():
 
-            return sorted(ee.Image(self.src_gee).bandNames().getInfo())
+            return ee.Image(self.src_gee).bandNames().getInfo()
 
         @su.need_ee
         def _ee_vector():
 
             columns = ee.FeatureCollection(self.src_gee).first().getInfo()["properties"]
 
-            return sorted(
+            return (
                 str(c)
                 for c in columns.keys()
                 if c not in ["system:index", "Shape_Area"]
@@ -204,7 +223,7 @@ class ReclassifyModel(Model):
             with rio.open(self.src_local) as f:
                 bands = [i for i in range(1, f.count + 1)]
 
-            return sorted(bands)
+            return bands
 
         def _local_vector():
 
@@ -217,18 +236,30 @@ class ReclassifyModel(Model):
 
         # return the selected function
         # remember to use self as a parameter
-        return band_func[self.gee][self.input_type]()
+        return natsorted(band_func[self.gee][self.input_type]())
 
     def get_aoi(self):
         """Validate and get feature collection from aoi_model"""
 
-        # TODO: Validate if the aoi is within the bounds of the inputs
+        # return None if no aoi_model is selected
+        if not self.aoi_model:
+            return
 
-        if self.aoi_model:
-            if not self.aoi_model.feature_collection:
+        # return none if no aoi is selected
+        # test gdf as it's set whatever the type of AoiModel (gee or not)
+        if self.aoi_model.gdf is None:
+            if self.enforce_aoi:
                 raise Exception("You have to select an area of interest before")
             else:
-                return self.aoi_model.feature_collection.geometry()
+                aoi = None
+
+        else:  # return the aoi as a vector
+            if self.gee:
+                aoi = self.aoi_model.feature_collection
+            else:
+                aoi = self.aoi_model.gdf
+
+        return aoi
 
     def unique(self):
         """
@@ -240,13 +271,20 @@ class ReclassifyModel(Model):
             and there color/name defaulted to none and black
         """
 
+        if not self.band:
+            raise Exception("You need to provide a band/property to reclassify.")
+
         @su.need_ee
         def _ee_image():
 
             # reduce the image
             image = ee.Image(self.src_gee).select(self.band)
-            geometry = image.geometry() if not self.aoi_model else self.get_aoi()
-            reduction = image.reduceRegion(ee.Reducer.frequencyHistogram(), geometry)
+            aoi = self.get_aoi() or image
+            geometry = aoi.geometry()
+
+            reduction = image.reduceRegion(
+                ee.Reducer.frequencyHistogram(), geometry, maxPixels=1e13
+            )
 
             # Remove all the unnecessary reducer output structure and make a
             # list of values.
@@ -259,50 +297,51 @@ class ReclassifyModel(Model):
         @su.need_ee
         def _ee_vector():
 
-            if self.aoi_model:
-                geometry = self.get_aoi()
+            collection = ee.FeatureCollection(self.src_gee)
+            aoi = self.get_aoi() or collection
+            geometry = aoi.geometry()
+
+            # get the feature
+            values = (
+                collection.filterBounds(geometry).aggregate_array(self.band).getInfo()
+            )
 
-                # get the feature
-                values = (
-                    ee.FeatureCollection(self.src_gee)
-                    .filterBounds(geometry)
-                    .aggregate_array(self.band)
-                    .getInfo()
-                )
-            else:
-                values = (
-                    ee.FeatureCollection(self.src_gee)
-                    .aggregate_array(self.band)
-                    .getInfo()
-                )
             return list(set(values))
 
         def _local_image():
 
             with rio.open(self.src_local) as src:
 
-                count = np.bincount(src.read(self.band).flatten())
+                bounds = self.get_aoi().total_bounds if self.get_aoi() else src.bounds
+                data = src.read(
+                    self.band, window=from_bounds(*bounds, transform=src.transform)
+                )
+                count = np.bincount(data.flatten())
+
             return np.nonzero(count != 0)[0].tolist()
 
         def _local_vector():
 
-            df = gpd.read_file(self.src_local)
+            gdf = gpd.read_file(self.src_local)
+            bounds = self.get_aoi().total_bounds if self.get_aoi() else gdf.total_bounds
+            xmin, ymin, xmax, ymax = bounds
+            gdf = gdf.cx[xmin:xmax, ymin:ymax]
 
-            return df[self.band].unique().tolist()
+            return gdf[self.band].unique().tolist()
 
         # map all the function in the guess matrix (gee, type)
         unique_func = [[_local_vector, _local_image], [_ee_vector, _ee_image]]
 
         # get values from the selected func
         # remember to use self as a parameter
-        values = unique_func[self.gee][self.input_type]()
+        values = natsorted(unique_func[self.gee][self.input_type]())
 
         # create the init dictionnary
         self.src_class = {v: ("no_name", "#000000") for v in values}
 
         return self.src_class
 
-    def reclassify(self, save=True):
+    def reclassify(self):
         """
         Reclassify the input according to the provided matrix. For vector file type reclassifying correspond to add an extra column at the end, for raster the initial class band will be replaced by the new class, the oher being kept unmodified. vizualization colors will be set for both local (QGIS compatible) and assets (SEPAL vizualization compatible).
 
@@ -310,18 +349,29 @@ class ReclassifyModel(Model):
             self
         """
 
+        if not self.matrix:
+            raise Exception(
+                "You need a reclassification matrix to reclassify an asset."
+            )
+        if not self.band:
+            raise Exception("You need to provide a band/property to reclassify.")
+
         @su.need_ee
         def _ee_image():
 
+            if not self.src_gee:
+                raise Exception("You need to provide source asset.")
+
             # create the asset description
-            self.dst_gee = Path(self.folder) / f"{Path(self.src_gee).stem}_reclass"
+            self.set_dst_gee()
 
             # load the image
             # remap according to the matrix
             from_, to_ = list(zip(*matrix.items()))
 
             image = ee.Image(self.src_gee)
-            geometry = image.geometry() if not self.aoi_model else self.get_aoi()
+            aoi = self.get_aoi() or image
+            geometry = aoi.geometry()
 
             ee_image = (
                 image.clip(geometry)
@@ -334,9 +384,11 @@ class ReclassifyModel(Model):
 
             # add colormapping parameters
             # set return an element so we force cast it to ee.Image
-            code, desc, color = zip(
-                *[(str(k), str(v[0]), str(v[1])) for k, v in self.dst_class.items()]
-            )
+            code, desc, color = ["0"], ["no_data"], ["#000000"]
+            for k, v in self.dst_class.items():
+                code.append(str(k))
+                desc.append(str(v[0]))
+                color.append(str(v[1]))
 
             ee_image = ee.Image(
                 ee_image.set(
@@ -344,9 +396,9 @@ class ReclassifyModel(Model):
                         "visualization_0_name": "Classification",
                         "visualization_0_bands": self.band,
                         "visualization_0_type": "categorical",
-                        "visualization_0_labels": f"no_data,{','.join(desc)}",
-                        "visualization_0_palette": f"#000000,{','.join(color)}",
-                        "visualization_0_values": f"0,{','.join(code)}",
+                        "visualization_0_labels": ",".join(desc),
+                        "visualization_0_palette": ",".join(color),
+                        "visualization_0_values": ",".join(code),
                     }
                 )
             )
@@ -354,13 +406,14 @@ class ReclassifyModel(Model):
             # save the file in a in_memeory variable
             self.dst_gee_memory = ee_image
 
-            if save:
+            if self.save:
+
                 # export
                 params = {
                     "image": ee_image,
-                    "assetId": str(self.dst_gee),
-                    "description": self.dst_gee.stem,
-                    "scale": 30,  # it should be the native resolution of the original img
+                    "assetId": self.dst_gee,
+                    "description": Path(self.dst_gee).stem,
+                    "scale": ee_image.projection().nominalScale().getInfo(),
                     "maxPixels": 1e13,
                     "pyramidingPolicy": {".default": "mode"},
                 }
@@ -368,13 +421,16 @@ class ReclassifyModel(Model):
                 task = ee.batch.Export.image.toAsset(**params)
                 task.start()
 
-            return self.dst_gee.stem
+            return self.dst_gee
 
         @su.need_ee
         def _ee_vector():
 
+            if not self.src_gee:
+                raise Exception("You need to provide source asset.")
+
             # create the asset description
-            self.dst_gee = Path(self.folder) / f"{Path(self.src_gee).stem}_reclass"
+            self.set_dst_gee()
 
             # add a new propertie
 
@@ -388,92 +444,107 @@ class ReclassifyModel(Model):
             ee_matrix = ee.List(list(matrix.items())).unzip()
             ee_from, ee_to = ee.List(ee_matrix.get(0)), ee.List(ee_matrix.get(1))
 
-            if self.aoi_model:
-                aoi_geometry = self.get_aoi()
-                self.dst_gee_memory = (
-                    ee.FeatureCollection(self.src_gee)
-                    .filterBounds(aoi_geometry)
-                    .map(add_prop)
-                )
-            else:
-                self.dst_gee_memory = ee.FeatureCollection(self.src_gee).map(add_prop)
+            collection = ee.FeatureCollection(self.src_gee)
+            aoi = self.get_aoi() or collection
+            geometry = aoi.geometry()
+
+            self.dst_gee_memory = (
+                ee.FeatureCollection(self.src_gee).filterBounds(geometry).map(add_prop)
+            )
 
             # add colormapping parameters
 
-            if save:
+            if self.save:
+
                 # export
                 params = {
                     "collection": self.dst_gee_memory,
-                    "assetId": str(self.dst_gee),
-                    "description": str(self.dst_gee.stem),
+                    "assetId": self.dst_gee,
+                    "description": Path(self.dst_gee).stem,
                 }
 
                 task = ee.batch.Export.table.toAsset(**params)
                 task.start()
 
-            return self.dst_gee.stem
+            return self.dst_gee
 
         def _local_image():
 
+            if not self.src_local:
+                raise Exception("You need to provide source asset.")
+
             # set the output file
-            self.dst_local = self.dst_dir / f"{Path(self.src_local).stem}_reclass.tif"
+            self.dst_local = str(
+                self.dst_dir / f"{Path(self.src_local).stem}_reclass.tif"
+            )
 
-            with rio.open(self.src_local) as src_f:
+            with rio.open(self.src_local) as src:
 
-                profile = src_f.profile
-                profile.update(driver="GTiff", count=1, compress="lzw", dtype=np.uint8)
+                bounds = self.get_aoi().total_bounds if self.get_aoi() else src.bounds
+                window = from_bounds(*bounds, transform=src.transform)
+                raw = src.read(self.band, window=window)
 
-                with rio.open(self.dst_local, "w", **profile) as dst_f:
+                data = np.zeros_like(raw, dtype=np.int64)
 
-                    # workon each window to avoid memory problems
-                    windows = [w for _, w in src_f.block_windows()]
-                    for window in windows:
+                for old_val, new_val in matrix.items():
+                    data += (raw == old_val) * new_val
 
-                        # read the window
-                        raw = src_f.read(self.band, window=window)
+                if self.save:
 
-                        # reclassify the image based on the matrix
-                        # every value that is not specified will be set to 0
-                        data = np.zeros_like(raw, dtype=np.int64)
+                    profile = src.profile
+                    profile.update(
+                        driver="GTiff",
+                        count=1,
+                        compress="lzw",
+                        dtype=np.uint8,
+                        height=window.height,
+                        width=window.width,
+                        transform=src.window_transform(window),
+                    )
 
-                        for old_val, new_val in matrix.items():
-                            data += (raw == old_val) * new_val
+                    with rio.open(self.dst_local, "w", **profile) as dst:
+                        dst.write(data.astype(np.uint8), 1, window=window)
+
+                        # add the colors to the image
+                        colormap = {0: (0, 0, 0)}
+                        for code, item in self.dst_class.items():
+                            colormap[code] = tuple(
+                                int(c * 255) for c in to_rgba(item[1])
+                            )
+                        dst.write_colormap(self.band, colormap)
 
-                        # write it in the destination file
-                        if save:
-                            dst_f.write(data.astype(np.uint8), 1, window=window)
                     # Save raster in memory
                     self.dst_local_memory = data
 
-                    # add the colors to the image
-                    colormap = {0: (0, 0, 0)}
-                    for code, item in self.dst_class.items():
-                        colormap[code] = tuple(int(c * 255) for c in to_rgba(item[1]))
-                    dst_f.write_colormap(self.band, colormap)
-
             return self.dst_local
 
         def _local_vector():
 
-            # set the output file
-            self.dst_local = self.dst_dir / f"{Path(self.src_local).stem}_reclass.shp"
+            if not self.src_local:
+                raise Exception("You need need to provide source asset.")
 
             # set the output file
-            self.dst_local = self.dst_dir / f"{Path(self.src_local).stem}_reclass.shp"
+            self.dst_local = str(
+                self.dst_dir / f"{Path(self.src_local).stem}_reclass.shp"
+            )
 
             # read the dataset
             gdf = gpd.read_file(self.src_local)
+            bounds = self.get_aoi().total_bounds if self.get_aoi() else gdf.total_bounds
+            xmin, ymin, xmax, ymax = bounds
+            gdf = gdf.cx[xmin:xmax, ymin:ymax]
 
             # map the new column
-            gdf["reclass"] = gdf.apply(lambda row: matrix[row[self.band]])
+            gdf["reclass"] = gdf.apply(lambda row: matrix[row[self.band]], axis=1)
 
             # add the colors to the gdf
             # waiting for an answer there :
             # https://gis.stackexchange.com/questions/404946/how-can-i-save-my-geopandas-symbology
             self.dst_local_memory = gdf
 
-            if save:
-                # save the file
+            # save the file
+            if self.save:
+
                 gdf.to_file(self.dst_local)
 
             return self.dst_local
@@ -489,7 +560,41 @@ class ReclassifyModel(Model):
         # remember to use self as a parameter
         res = reclassify_func[self.gee][self.input_type]()
 
-        # tel the rest of the apps that a reclassification is finished
+        # tell the rest of the apps that a reclassification is finished
         self.remaped += 1
 
-        return ms.rec.rec.export[self.gee][self.input_type].format(res)
+        if self.save:
+            return ms.rec.rec.export[self.gee][self.input_type].format(res)
+
+        return "Asset successfully reclassified."
+
+    def set_dst_gee(self):
+        """
+        Creates a unique and consecutive asset name based on the source
+
+        Return:
+            (str) the destination folder
+        """
+
+        # create the asset_id
+        asset_name = f"{Path(self.src_gee).stem}_reclass"
+
+        # impossible to guess the folder as an asset name can start with:
+        # project/earthengin-legacy/users/xx/xx/....
+        # users/xx/xx/xx
+        # OXFORD/xx/xx/xx for public dataset
+        # fallback to the root folder
+        dst_gee = str(Path(self.folder, asset_name))
+
+        # check if the name already exist
+        current_assets = [asset["name"] for asset in gee.get_assets(self.folder)]
+
+        # An user could reclassify twice an asset,
+        # So let's create an unique name
+        while dst_gee in current_assets:
+            dst_gee = su.next_string(dst_gee)
+
+        # set the dst_gee of the model
+        self.dst_gee = dst_gee
+
+        return self
