@@ -16,6 +16,8 @@ from matplotlib import colors as c
 
 import sepal_ui
 
+from .warning import SepalWarning
+
 
 def hide_component(widget):
     """
@@ -251,17 +253,39 @@ def loading_button(alert=None, button=None, debug=False):
             value = None
             try:
                 # Catch warnings in the process function
-                with warnings.catch_warnings(record=True) as w:
+                with warnings.catch_warnings(record=True) as w_list:
                     value = func(self, *args, **kwargs)
 
                 # Check if there are warnings in the function and append them
-                # Use append msg due to several warnings could be triggered
-                if w:
-                    [
-                        alert_.append_msg(warning.message.args[0], type_="warning")
-                        for warning in w
+                # Use append msg as several warnings could be triggered
+                if w_list:
+
+                    # split the warning list
+                    w_list_sepal = [
+                        w for w in w_list if isinstance(w.message, SepalWarning)
                     ]
 
+                    # display the sepal one
+                    ms_list = [
+                        f"{w.category.__name__}: {w.message.args[0]}"
+                        for w in w_list_sepal
+                    ]
+                    [alert_.append_msg(ms, type_="warning") for ms in ms_list]
+
+                    # only display them in the console if debug mode
+                    if debug:
+
+                        def custom_showwarning(w):
+                            return warnings.showwarning(
+                                message=w.message,
+                                category=w.category,
+                                filename=w.filename,
+                                lineno=w.lineno,
+                                line=w.line,
+                            )
+
+                        [custom_showwarning(w) for w in w_list]
+
             except Exception as e:
                 alert_.add_msg(f"{e}", "error")
                 if debug:
